<?php
// +----------------------------------------------------------------------
// | 使用注解，实现依赖注入
// +----------------------------------------------------------------------
// | Copyright (c) 2018 https://blog.junphp.com All rights reserved.
// +----------------------------------------------------------------------
// | Licensed ( http://www.apache.org/licenses/LICENSE-2.0 )
// +----------------------------------------------------------------------
// | Author: 小黄牛 <1731223728@qq.com>
// +----------------------------------------------------------------------


/**
 * 注解说明：
 * @Ioc(class="类的路径，例如：\think\Db::name()", function="", name="test", args="1,2,3")  
 * 容器注入：
 * class=容器的地址
 * function=容器的执行方法
 * name=要绑定的成员属性名称
 * args=需要传入function的参数
*/

require_once 'vendor/App.php'; // 应用类
require_once 'vendor/Annotate.php'; // 注解类
require_once 'vendor/DocParser.php'; // 注解解析类

echo '<pre>';
App::run('Test', 'A', '小黄牛', '真帅~');

/**
 * 假设我是一个 静态的Test控制器
 * 静态的容器注入，需要提前声明被注入的成员属性，并且需要是public类型的，否则无法注入
 * @Ioc(class="Demo", name="all")
*/
class Test {
    /**
     * 全局容器
    */
    public static $all;

    /**
     * 局部容器
    */
    public static $local;

    /**
     * 尼妹的函数
     * @Ioc(class="Demo", name="local", function="_echo", args="'1,1',2,'1,3,0'")
     * @param string $name 打死你
     * @return void
    */
    public static function A($a=0, $b=1) {
        // 调用全局容器
        var_dump(self::$all->_echo(4,5,6));
        // 调用局部容器
        var_dump(self::$local);
        // 调用自身变量
        var_dump($a, $b);
    }
}

/**
 * 假设我是一个 动态的Test2控制器
 * @Ioc(class="Demo", name="all")
*/
class Test2 {

    /**
     * 尼妹的函数
     * @Ioc(class="Demo", name="local", function="_echo", args="'1,1',2,'1,3,0'")
     * @param string $name 打死你
     * @return void
    */
    public function A($a=0, $b=1) {
        // 调用全局容器
        var_dump($this->all->_echo(4,5,6));
        // 调用局部容器
        var_dump($this->local);
        // 调用自身变量
        var_dump($a, $b);
    }
}


// 而我是需要被注入的容器
class Demo {
    public function _echo ($a, $b, $c) {
        return [$a, $b, $c];
    }
}