<?php
// +----------------------------------------------------------------------
// | 注解读取类
// +----------------------------------------------------------------------
// | Copyright (c) 2018 https://blog.junphp.com All rights reserved.
// +----------------------------------------------------------------------
// | Licensed ( http://www.apache.org/licenses/LICENSE-2.0 )
// +----------------------------------------------------------------------
// | Author: 小黄牛 <1731223728@qq.com>
// +----------------------------------------------------------------------

class Annotate {
    private $Reflection;
    private $instance;
    private $method;

    /**
     * 启动注解解析
     * @todo 无
     * @author 小黄牛
     * @version v1.0.1 + 2020.05.25
     * @deprecated 暂不启用
     * @global 无
     * @param string $class 对应的控制器
     * @param mixed $action 对应的操作
     * @param array $args 操作的参数
     * @return void
    */
    public function run($class, $action, ...$args) {
        if (!class_exists($class)) die('该类不存在');

        # 反射方法
        $method = new \ReflectionMethod($class, $action);

        # 不是公共方法，不给执行
        if (!$method->isPublic()) {
            die("{$class}::{$action}不是public方法");
        }

        # 使用ReflectionClass类
        $this->Reflection = new \ReflectionClass($class);

        # 解析注解
        $doc = DocParserFactory::getInstance()->parse($this->getClass());

        # 通过反射类进行实例化
        $this->instance = $this->Reflection->newInstance();
        # 获取方法的实例-但未执行
        $this->method = $this->Reflection->getmethod($action);

        # 注入全局容器
        $this->Ioc($doc);
        
        # 局部的注解
        $doc = DocParserFactory::getInstance()->parse($this->getFunction($action));
        # 注入局部容器
        $this->Ioc($doc);

        # 调用对应的方法
        $this->method->invokeArgs($this->instance, $args);
    }

    // 容器注入
    private function Ioc($doc) {
        # 解析容器注入 - 因为是Controller等于全局容器注入
        foreach ($doc as $val) {
            foreach ($val as $key=>$value) {
                # IOC容器注入
                if ($key == 'Ioc') {
                    # 这里你可能还要检查注入的类、方法是否存在
                    # name指定的属性名称是否被冲突

                    $args = [];
                    if (!empty($value['args'])) {
                        $args = $value['args'];
                    }

                    $name = $value['name'];
                    $reflection = new \ReflectionClass($value['class']);
                    $obj = $reflection->newInstance(); 

                    # 动态属性注入
                    if (!$this->method->isStatic()) {
                        if ($value['function']) {
                            $method = $reflection->getmethod($value['function']);
                            $this->instance->$name = $method->invokeArgs($obj, $args);
                        } else {
                            $this->instance->$name = $obj;
                        }
                    # 静态属性注入
                    } else {
                        if ($value['function']) {
                            $method = $reflection->getmethod($value['function']);
                            $this->Reflection->setStaticPropertyValue($name, $method->invokeArgs($obj, $args));
                        } else {
                            $this->Reflection->setStaticPropertyValue($name, $obj);
                        }
                    }
                }
            }
        }
    }
    
    // 获得类对应的注解
    public function getClass() {
        return $this->Reflection->getDocComment();
    }

    // 获得指定方法对应的注解
    public function getFunction($action) {
        # 获得成员函数名称
        $methods = $this->Reflection->getMethods();

        foreach ($methods as $method) {
            # 只要指定方法的
            if ($method->name == $action) {
                # 获得成员函数对应的注释内容
                return $method->getDocComment();
            }
        }
        return false;
    }
}
